import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;

/* 
 * To jest klasa z wykladu nr 4 z drobnymi zmianami, zeby eclipse nie wywalal 'ostrzezen'
 * --------------------
 * klasa zrobina przez wykladowce!, wiec raczej nie powinien sie czepiac
 * ale trzeba rozumiec KAZDY szczegol jej dzialania!
 * */
public class ObjectPlus implements Serializable
{
	/*
	wymagane do serializacji, w przyszlosci jak cos zmienisz w tej klasie [dodasz/usuniesz atrybut]
	to zmienisz ta liczbe na inna, dzieki temu mechanizm 'serialize' nie pozwoli zaladowac 
	pliku z dysku z starymi danymi do programu ktory uzywa innej klasy
	(przeciez moze byc zupelnie inna klasa o nazwie Student i dane takie jak 'oceny' moga w niej
	nie wystepowac)
	 */
	private static final long serialVersionUID = 1L;
	/*
	(w tym tlumaczeniu bede uzywal slowa 'tablica' zamiast 'lista', roznica? lista [z ktorej korzystamy pod nazwa Hashtable!]
	sama 'rosnie' jak do niej dodajemy obiekty, a tablice maja staly rozmiar np. 'new Integer[5]' - 5 elementow)
	
	Tlumaczenie typow:
	1. Class<? extends ObjectPlus> - obiekt typu Class pobrany z obiektu ktory rozszerza ObjectPlus, czyli np. Osoba lub Student,
	String czy Integer nie rozszerzaja ObjectPlus, wiec nie zostana zaakceptowane
	2. Vector<ObjectPlus> - lista/kontener obiektow typu ObjectPlus, do takiej listy mozna wsadzic dowolny obiekt
	ktory rozszerza ta klase, czyli np. Student czy Osoba
	
	Opis:
	To jest 'tablica tablic', a dokladniej tablica ktora jako klucz nie ma liczby, a.. KLASE (Class)!
	Class to o dziwo w Javie klasa ktora przechowuje informacje o klasie, mozna pobrac obiekt typu Class z kazdego obiektu
	w Javie uzywajac:
	Class klasaObiektu = mojObiekt.getClass();
	No wiec mamy juz tablice ktora uzywa jako klucza Class, a jakie dane przechowuje? Przechowuje Vector<ObjectPlus>,
	czyli tablice obiektow typu ObjectPlus:
	1. Osoba rozszerza ObjectPlus [extends], wiec tez moga ja wrzucic do tej tablicy
	2. Student rozszerza Osoba ktora rozszerza ObjectPlus, wiec tez moge ja tam wrzucic
	
	A po co mi wogole tablica tablic zamiast zwyklej tablicy?
	Bo ObjectPlus w projekcie docelowym moze byc rozszerzany przez wiele obiektow roznych
	klas jak np. 'Wplata', 'Ocena', 'Przedmiot' itd., a nie chce mi sie pisac nowego ObjectPlus dla kazdej z nich
	Nie chce tez przechowywac wszystkich obiektow w jednej tablicy [typu Object, wtedy moge tam wrzucic wszystko],
	bo wtedy 'pobranie wszystkich 'Student' wymagalo by przegladania wszystkich elementow tablicy i
	sprawdzania ktorzy to Studenci.
	 */
	private static Hashtable<Class<? extends ObjectPlus>, Vector<ObjectPlus>> ekstensje = new Hashtable<Class<? extends ObjectPlus>, Vector<ObjectPlus>>();

	/**
	 * Konstruktor.
	 */
	public ObjectPlus()
	{
		Vector<ObjectPlus> ekstensja = null;
		Class<? extends ObjectPlus> klasa = this.getClass();

		if(ekstensje.containsKey(klasa))
		{
			// Ekstensja tej klasy istnieje w kolekcji ekstensji
			// jest juz jakis np. Student, wiec jest i lista studentow, nie chce tworzyc nowej listy,
			// tylko dodac nowego studenta do listy juz istniejacej
			ekstensja = ekstensje.get(klasa);
		}
		else
		{
			// Ekstensji tej klasy jeszcze nie ma -> dodaj j
			// to wystepuje kiedy pierwszy raz dodajemy obiekt danego typu (np. Student) do ekstensji
			ekstensja = new Vector<ObjectPlus>();
			ekstensje.put(klasa, ekstensja);
		}
		// a tu wreszcie dodaje studenta do listy nowej LUB istniejacej
		ekstensja.add(this);
	}
	
	/**
	 * Zapisuje wszystkie ekstensje do podanego strumienia (wersja z serializacja).
	 * Metoda klasowa.
	 * @throws IOException 
	 */
	public static void zapiszEkstensje(ObjectOutputStream stream) throws IOException
	{
		/*
		'magia javy' zamienia obiekt (Hashmape) zawierajacy inne obiekty w tekst,
		wiec jak zawiera liste Studentow to Ci studenci maja np. atrybut 'Date dataUrodzenia' i to tez sie zapisze
		(Date jest 'serializable' jak wiekszosc klas uzywanych na codzien w javie)
		(wszystkie obiekty jakie zawiera ekstensja musza rozszerzac interfejs 'Serializable')
		java znajdzie wszystkie powiazane obiekty z tymi obiektami ktore mamy w hashmap 'ekstensje' i 
		wszystkie powiazane z ich powiazaniami itd. i wszystko zapisze jako tekst!
		a potem jeszcze moze z tego tekstu odtworzyc ten obiekt [i obiekty powiazane], magia!
		
		UWAGA: to zapisze tylko zmienne 'obiektu', zmienne 'klasowe' (te z 'static') nie zostana zapisane i
		to jeden z problemow z takim prostym zapisem, ale mozna samemu dopisac zapis 'static'
		na szczescie to nie jest wymagane w MP_1, a jedynie w projekcie semestralnym
		*/
		stream.writeObject(ekstensje);
	}

	/**
	 * Odczytuje wszystkie ekstensje z podanego strumienia (wersja z serializacja).
	 * Metoda klasowa.
	 * @throws IOException 
	 * @throws ClassNotFoundException 
	 */
	@SuppressWarnings("unchecked")
	public static void odczytajEkstensje(ObjectInputStream stream) throws IOException, ClassNotFoundException
	{
		/*
		tutaj 'magia javy' odtwarza dane zapisane jako tekst do pliku (zapisane w funkcji wyzej)
		 */
		ekstensje = (Hashtable<Class<? extends ObjectPlus>, Vector<ObjectPlus>>) stream.readObject();
	}	
	
	/**
	 * Wyswietla ekstensje.
	 * Metoda klasowa.
	 * @throws Exception 
	 */
	public static void pokazEkstensje(Class<? extends ObjectPlus> klasa) throws Exception
	{
		/*
		wystarczy gdzies w programie wywolac ta funkcje tak:
		ObjectPlus.pokazEkstensje(Student.class);
		aby wyswietlic wszystkie obiekty typu 'Student' istniejace w programie
		 */
		Vector<ObjectPlus> ekstensja = null;
		
		if(ekstensje.containsKey(klasa))
		{
			// Ekstensja tej klasy istnieje w kolekcji ekstensji
			ekstensja = ekstensje.get(klasa);
		}
		else
		{
			throw new Exception("Unknown class " + klasa);
		}
		
		System.out.println("Ekstensja klasy: " + klasa.getSimpleName());

		for(Object obiekt : ekstensja)
		{
			System.out.println(obiekt + "\n");
		}
	}
	
}
