import java.util.Date;


public abstract class Osoba extends ObjectPlus
{
	/*
	wymagane do serializacji, w przyszlosci jak cos zmienisz w tej klasie [dodasz/usuniesz atrybut]
	to zmienisz ta liczbe na inna, dzieki temu mechanizm 'serialize' nie pozwoli zaladowac 
	pliku z dysku z starymi danymi do programu ktory uzywa innej klasy
	(przeciez moze byc zupelnie inna klasa o nazwie Student i dane takie jak 'oceny' moga w niej
	nie wystepowac)
	 */
	private static final long serialVersionUID = 1L;
	/* PRZYPOMNIENIE: Co to jest 'protected'?
	 * To takie cos jak 'private' tylko, ze zmienna jest widziana w klasach 'dziedziczacych' (rozszerzajacych).
	 * Dzieki temu bedzie mozna wyswietlac np. 'imie' w studencie ktory dziedziczy po osobie.
	 */
	
	// atrybut prosty, wymagany, pojedynczy, obiektu
	protected String imie;
	/*
	 * WYMAGANY:
	 * zawsze kiedy go ustawiamy to sprawdzamy czy ktos nam nie przekazal 'null'
	 */
	
	// atrybut prosty, opcjonalny, pojedynczy, obiektu
	protected String nazwisko;
	/*
	 * OPCJONALNY:
	 * zawsze kiedy go wyswietlany to sprawdzany cyz ktos nam nie przekazal 'null'
	 * jesli 'null' to trzeba wyswietlic o tym informacje lub inaczej to 'obsluzyc',
	 * bo proba wyswietlenie 'null' skonczy sie bledem programu
	 */
	
	// atrybut zlozony, opcjonalny, pojedynczy, obiektu
	protected Date dataUrodzenia;
	/*
	 * ZLOZONY:
	 * to taki ktory zawiera w sobie wiecej niz jedna informacje
	 * ogolnie chodzi o odniesienie sie do innego obiektu (oprocz 'String')
	 * 
	 */
	// atrybut prosty, wymagany, pojedynczy, obiektu
	protected int wzrost = 0;
	/*
	 * jest tu tylko po to, zeby wyliczac srednia (i miec cos 'pochodnego')
	 */
	
	// atrybut prosty, pojedynczy, wyliczalny (pochodny), klasowy
	private static double sredniWzrost = 0;
	/*
	 * KLASOWY:
	 * dotyczy klasy, czyli wszystkich 'Osob', a nie jednej konkretnej
	 * technicznie: zmienna typu static
	 * 
	 * POCHODNY:
	 * mozna go wyliczyc, w tym wypadku wystarczylo by pobrac wszystkie Osoby z 'ekstensji' ObjectPlus
	 * pobrac ich wzrost i wyliczyc srednia, ale prosciej to po prostu pamietac
	 * i liczyc tylko raz przy dodawaniu osoby
	 */
	// atrybut prosty, pojedynczy, klasowy - potrzebny do liczenia sredniej
	private static int iloscOsob = 0;
	
	// konstruktor nie jest 'pusty', bo imieOsoby jest 'wymagane', wiec musi byc w kazdym konstruktorze
	Osoba(String imieOsoby, int wzrostOsoby)
	{
		/*
		 * BARDZO WAZNE: kazdy konstruktor klasy dziedziczacej po klasie ObjectPlus musi wywolywac 'super()'
		 * Co to jest 'super()'?
		 * Jest to cos co odpali konstruktor w klasie 'wyzej' / 'u rodzica' czyli w ObjectPlus.
		 * A co to robi?
		 * To zapewnia, ze kazdy utworzony obiekt jest na liscie 'ekstensji' w ObjectPlus.
		 * Konstruktor w ObjectPlus zajmuje sie dodawaniem/tworzeniem list ekstensji.
		 */
		super();

		
		
		// tutaj dbamy, aby wymagany atrybut byl ustawiony, a nie wskazywal na 'null'
		if(imieOsoby == null)
		{
			throw new NullPointerException("Osoba musi miec imie");
		}
		this.imie = imieOsoby;
		
		// 'int' nie moze byc 'nullem', wiec tego nie sprawdzamy
		// ale wymagamy podania tego w konstruktorze, wiec kazda osoba bedzie miala wpisany wzrost
		this.wzrost = wzrostOsoby;
		
		
		
		// to nam sie przyda do liczenia sredniej
		iloscOsob++;
		// a tu troche Drabikologi i mamy srednia
		sredniWzrost += (this.wzrost - sredniWzrost) / iloscOsob;
	}

	/*
	 * PRZESLANIANIE METOD:
	 * w nadklasie Osoba metoda ta zwraca tylko imie,
	 * a w Student imie z dodanym na poczatku tekstem 'Student'
	 * jaki tego sens? trudno mi wymyslic jakas sensowna przeslaniana funkcje
	 * 
	 * technicznie: w klasie dziedziczacej jest metoda o takiej samej nazwie, takich samych argumentach,
	 * ale zwraca cos innego
	 */
	public String getImie()
	{
		return this.imie;
	}
	
	/*
	 * METODA KLASOWA:
	 * metoda operujaca na zmiennych klasowych, a nie obiekcie
	 * technicznie: typu static
	 */
	public static double getSredniWzrost()
	{
		return sredniWzrost;
	}

	/*
	 * demonstracja roznicy w poslugiwaniu sie atrybutami 'wymaganymi' i 'opcjonalnymi'
	 */
	public void ustawImie(String imie)
	{
		if(imie == null) // uwaga wymagany!
		{
			throw new NullPointerException("Osoba musi miec imie");
		}
		this.imie = imie;
	}
	
	public void ustawNazwisko(String nazwisko)
	{
		this.nazwisko = nazwisko;
	}
	
	public void ustawDateUrodzenia(Date dataUrodzenia)
	{
		this.dataUrodzenia = dataUrodzenia;
	}
	
	public String toString()
	{
		String ret = new String();
		ret += "Imie: " + getImie();
		ret += "\nNazwisko: ";
		if(nazwisko != null) // uwaga opcjonalny!
			ret += nazwisko;
		else
			ret += "bez nazwiska";

		ret += "\nWzrost: " + wzrost;

		ret += "\nData urodzenia: ";
		if(dataUrodzenia != null) // uwaga opcjonalny!
			ret += dataUrodzenia;
		else
			ret += "nie znana";
		return ret;
	}
}
